@@ -30,20 +30,38 @@ def query_posthook(instance, data, request):
     data['_links'] = {
         'self': {
             'href': uri(reverse('serrano:queries:single', args=[instance.pk])),
+        },
+        'forks': {
+            'href': uri(reverse('serrano:queries:forks', args=[instance.pk])),
         }
     }
 
-    data['is_owner'] = instance.user == request.user
+    if getattr(instance, 'user', None) and instance.user.is_authenticated():
+        data['is_owner'] = instance.user == request.user
+    else:
+        data['is_owner'] = instance.session_key == request.session.session_key
 
-    # If this user is not the owner then the shared users are not included
-    # because this sort of data should not be exposed to anyone who isn't the
-    # owner of the original query.
     if not data['is_owner']:
         del data['shared_users']
 
     return data
 
 
+def forked_query_posthook(instance, data, request):
+    uri = request.build_absolute_uri
+    data['_links'] = {
+        'self': {
+            'href': uri(reverse('serrano:queries:single', args=[instance.pk])),
+        },
+        'parent': {
+            'href': uri(reverse('serrano:queries:single',
+                        args=[instance.parent.pk])),
+        }
+    }
+
+    return data
+
+
 class QueryBase(ThrottledResource):
     cache_max_age = 0
     private_cache = True
@@ -60,7 +78,7 @@ class QueryBase(ThrottledResource):
     def get_queryset(self, request, **kwargs):
         "Constructs a QuerySet for this user or session."
 
-        if hasattr(request, 'user') and request.user.is_authenticated():
+        if getattr(request, 'user', None) and request.user.is_authenticated():
             kwargs['user'] = request.user
         elif request.session.session_key:
             kwargs['session_key'] = request.session.session_key
@@ -84,12 +102,14 @@ class QueriesResource(QueryBase):
         return serialize(instance, posthook=posthook, **template)
 
     def get_queryset(self, request, **kwargs):
-        if hasattr(request, 'user') and request.user.is_authenticated():
+        if getattr(request, 'user', None) and request.user.is_authenticated():
             f = Q(user=request.user) | Q(shared_users__pk=request.user.pk)
-            return self.model.objects.filter(**kwargs).filter(f) \
-                .order_by('-accessed').distinct()
+        elif request.session.session_key:
+            f = Q(session_key=request.session.session_key)
         else:
             return super(QueriesResource, self).get_queryset(request, **kwargs)
+        return self.model.objects.filter(f, **kwargs) \
+            .order_by('-accessed').distinct()
 
     def get(self, request):
         queryset = self.get_queryset(request)
@@ -109,6 +129,118 @@ class QueriesResource(QueryBase):
         return response
 
 
+class QueryForksResource(QueryBase):
+    "Resource for accessing forks of the specified query or forking the query"
+    template = templates.ForkedQuery
+
+    def post(self, request, **kwargs):
+        if self.requestor_can_fork(request):
+            fork = DataQuery(name=request.instance.name,
+                             description=request.instance.description,
+                             view_json=request.instance.view_json,
+                             context_json=request.instance.context_json,
+                             parent=request.instance)
+
+            if getattr(request, 'user', None):
+                fork.user = request.user
+            elif request.session.session_key:
+                fork.session_key = request.session.session_key
+
+            fork.save()
+
+            posthook = functools.partial(query_posthook, request=request)
+            data = serialize(fork, posthook=posthook, **templates.Query)
+
+            return self.render(request, data, status=codes.created)
+
+        else:
+            return HttpResponse(status=codes.unauthorized)
+
+    def get_object(self, request, pk=None, *kwargs):
+        if not pk:
+            raise ValueError('A pk must be used for the fork lookup')
+
+        try:
+            return self.model.objects.get(pk=pk)
+        except self.model.DoesNotExist:
+            pass
+
+    def is_not_found(self, request, response, **kwargs):
+        instance = self.get_object(request, **kwargs)
+        if instance is None:
+            return True
+        request.instance = instance
+
+    def prepare(self, request, instance, template=None):
+        if template is None:
+            template = self.template
+
+        posthook = functools.partial(forked_query_posthook, request=request)
+        return serialize(instance, posthook=posthook, **template)
+
+    def get_queryset(self, request, **kwargs):
+        kwargs['parent'] = request.instance.pk
+
+        return self.model.objects.filter(**kwargs)
+
+    def requestor_can_get_forks(self, request):
+        """
+        A user can retrieve the forks of a query if that query is public or
+        if they are the owner of that query.
+        """
+        if request.instance.public:
+            return True
+
+        if not getattr(request, 'user', None):
+            return False
+
+        return (request.user.is_authenticated() and
+                request.user == request.instance.user)
+
+    def requestor_can_fork(self, request):
+        """
+        A user can fork a query if that query is public or if they are the
+        owner or in the shared_users group of that query.
+        """
+        if request.instance.public:
+            return True
+
+        if getattr(request, 'user', None) and request.user.is_authenticated():
+            return (request.user == request.instance.user or
+                    request.instance.shared_users
+                    .filter(pk=request.user.pk).exists())
+
+        return False
+
+    def get(self, request, **kwargs):
+        if self.requestor_can_get_forks(request):
+            return self.prepare(request, self.get_queryset(request))
+        else:
+            return HttpResponse(status=codes.unauthorized)
+
+
+class PublicQueriesResource(QueryBase):
+    "Resource for accessing public queries"
+    template = templates.BriefQuery
+
+    def prepare(self, request, instance, template=None):
+        if template is None:
+            template = self.template
+
+        posthook = functools.partial(query_posthook, request=request)
+        return serialize(instance, posthook=posthook, **template)
+
+    def get_queryset(self, request, **kwargs):
+        kwargs['public'] = True
+
+        return self.model.objects.filter(**kwargs).order_by('-accessed') \
+            .distinct()
+
+    def get(self, request):
+        queryset = self.get_queryset(request)
+        return self.prepare(request, queryset)
+
+
 class QueryResource(QueryBase):
     "Resource for accessing a single query"
     def get_object(self, request, pk=None, session=None, **kwargs):
@@ -166,6 +298,9 @@ class QueryResource(QueryBase):
 
 single_resource = never_cache(QueryResource())
 active_resource = never_cache(QueriesResource())
+public_resource = never_cache(PublicQueriesResource())
+forks_resource = never_cache(QueryForksResource())
+
 revisions_resource = never_cache(RevisionsResource(
     object_model=DataQuery, object_model_template=templates.Query,
     object_model_base_uri='serrano:queries'))
@@ -182,8 +317,10 @@ urlpatterns = patterns(
     url(r'^$', active_resource, name='active'),
 
     # Endpoints for specific queries
+    url(r'^public/$', public_resource, name='public'),
     url(r'^(?P<pk>\d+)/$', single_resource, name='single'),
     url(r'^session/$', single_resource, {'session': True}, name='session'),
+    url(r'^(?P<pk>\d+)/forks/$', forks_resource, name='forks'),
 
     # Revision related endpoints
     url(r'^revisions/$', revisions_resource, name='revisions'),
